/***************************************************************************
* Superpro.CS
*
* (C) Copyright 1991-2002 Rainbow Technologies, Inc. All rights reserved.
*
* Description - SuperPro file provide defination and wrappers for SSP dual Library
*
****************************************************************************/
using System.Runtime.InteropServices;
using System;

/* Set Protocol falgs */
public enum PROTOCOL_FLAG
{
	NSPRO_TCP_PROTOCOL         = 1,
	NSPRO_IPX_PROTOCOL         = 2,
	NSPRO_NETBEUI_PROTOCOL     = 4,
	NSPRO_SAP_PROTOCOL         = 8
}
/* Enum server falgs */
public enum ENUM_SERVER_FLAG
{
    NSPRO_RET_ON_FIRST =           1,
	NSPRO_GET_ALL_SERVERS =        2,
	NSPRO_RET_ON_FIRST_AVAILABLE = 4
}

public class NSPRO_SERVER_INFO 
{
	public byte[] serverAddress;
	public ushort numLicAvail;

	public NSPRO_SERVER_INFO()
	{
		serverAddress = new byte[Superpro.MAX_ADDR_LEN];
		numLicAvail = 0;	
	}

	public void GetServerInfo(byte[] bBuffer, int offset) 
	{
		int j = 0;
		for (int i=offset;i<(Superpro.MAX_ADDR_LEN+offset);i++)
			serverAddress[j++] = bBuffer[i];
		this.numLicAvail = (ushort)BitConverter.ToUInt16(bBuffer, Superpro.MAX_ADDR_LEN+offset);
	}
}

public class NSPRO_KEY_MONITOR_INFO {
	public ushort developerId;
	public ushort hardLimit;
	public ushort inUse;
	public ushort numTimeOut;
	public ushort highestUse;
}

public class NSPRO_MONITOR_INFO {
	public byte[] serverName;
	public byte[] serverIPAddress;
	public byte[] serverIPXAddress;
	public byte[] version;
	public ushort protocol;
	public NSPRO_KEY_MONITOR_INFO sproKeyMonitorInfo;

	public NSPRO_MONITOR_INFO()
	{
		this.serverName = new byte[Superpro.MAX_NAME_LEN];
		this.serverIPAddress = new byte[Superpro.MAX_ADDR_LEN];
		this.serverIPXAddress = new byte[Superpro.MAX_ADDR_LEN];
		this.version = new byte[Superpro.MAX_NAME_LEN];
		this.protocol = 0;
		this.sproKeyMonitorInfo = new NSPRO_KEY_MONITOR_INFO();
		this.sproKeyMonitorInfo.developerId = 0;
		this.sproKeyMonitorInfo.hardLimit = 0;
		this.sproKeyMonitorInfo.inUse = 0;
		this.sproKeyMonitorInfo.numTimeOut = 0;
		this.sproKeyMonitorInfo.highestUse = 0;
	}

	public void GetKeyInfo(byte[] bBuffer)
	{
		int index;
		int j = 0;
		int offset = 0;
		for (index=0;index<Superpro.MAX_NAME_LEN;index++)
			this.serverName[j++] = bBuffer[index];
		
		offset = index;
		j=0;
		for (index = offset;index<(Superpro.MAX_ADDR_LEN+offset);index++)
			this.serverIPAddress[j++] = bBuffer[index];
		
		offset = index;
		j=0;
		for (index = offset;index<(Superpro.MAX_ADDR_LEN+offset);index++)
			this.serverIPXAddress[j++] = bBuffer[index];

		offset = index;
		j=0;
		for (index = offset;index<(Superpro.MAX_NAME_LEN+offset);index++)
			this.version[j++] = bBuffer[index];

		this.protocol = (ushort)BitConverter.ToInt16(bBuffer, index);
		this.sproKeyMonitorInfo.developerId = (ushort)BitConverter.ToUInt16(bBuffer, index+2);
		this.sproKeyMonitorInfo.hardLimit = (ushort)BitConverter.ToUInt16(bBuffer, index+4);
		this.sproKeyMonitorInfo.inUse = (ushort)BitConverter.ToUInt16(bBuffer, index+6);
		this.sproKeyMonitorInfo.numTimeOut = (ushort)BitConverter.ToUInt16(bBuffer, index+8);
		this.sproKeyMonitorInfo.highestUse = (ushort)BitConverter.ToUInt16(bBuffer, index+10);
	}


}

public class Superpro 
{
	//Error code for SSP Library
	public const int SP_SUCCESS						= 0;
	public const int SP_INVALID_FUNCTION_CODE		= 1;
	public const int SP_INVALID_PACKET				= 2;
	public const int SP_UNIT_NOT_FOUND				= 3;
	public const int SP_ACCESS_DENIED				= 4;
	public const int SP_INVALID_MEMORY_ADDRESS		= 5;
	public const int SP_INVALID_ACCESS_CODE			= 6;
	public const int SP_PORT_IS_BUSY				= 7;
	public const int SP_WRITE_NOT_READY				= 8;
	public const int SP_NO_PORT_FOUND				= 9;
	public const int SP_ALREADY_ZERO				= 10;
	public const int SP_DRIVER_OPEN_ERROR			= 11;
	public const int SP_DRIVER_NOT_INSTALLED		= 12;
	public const int SP_IO_COMMUNICATIONS_ERROR		= 13;
	public const int SP_PACKET_TOO_SMALL			= 15;
	public const int SP_INVALID_PARAMETER			= 16;
	public const int SP_MEM_ACCESS_ERROR			= 17;
	public const int SP_VERSION_NOT_SUPPORTED		= 18;
	public const int SP_OS_NOT_SUPPORTED			= 19;
	public const int SP_QUERY_TOO_LONG				= 20;
	public const int SP_INVALID_COMMAND				= 21;
	public const int SP_MEM_ALIGNMENT_ERROR			= 29;
	public const int SP_DRIVER_IS_BUSY				= 30;
	public const int SP_PORT_ALLOCATION_FAILURE		= 31;
	public const int SP_PORT_RELEASE_FAILURE		= 32;
	public const int SP_ACQUIRE_PORT_TIMEOUT		= 39;
	public const int SP_SIGNAL_NOT_SUPPORTED		= 42;
	public const int SP_UNKNOWN_MACHINE				= 44;
	public const int SP_SYS_API_ERROR				= 45;
	public const int SP_UNIT_IS_BUSY				= 46;
	public const int SP_INVALID_PORT_TYPE			= 47;
	public const int SP_INVALID_MACH_TYPE			= 48;
	public const int SP_INVALID_IRQ_MASK			= 49;
	public const int SP_INVALID_CONT_METHOD			= 50;
	public const int SP_INVALID_PORT_FLAGS			= 51;
	public const int SP_INVALID_LOG_PORT_CFG		= 52;
	public const int SP_INVALID_OS_TYPE				= 53;
	public const int SP_INVALID_LOG_PORT_NUM		= 54;
	public const int SP_INVALID_ROUTER_FLGS			= 56;
	public const int SP_INIT_NOT_CALLED				= 57;
	public const int SP_DRVR_TYPE_NOT_SUPPORTED		= 58;
	public const int SP_FAIL_ON_DRIVER_COMM			= 59;
	public const int SP_SERVER_PROBABLY_NOT_UP		= 60;
	public const int SP_UNKNOWN_HOST				= 61;
	public const int SP_SENDTO_FAILED				= 62;
	public const int SP_SOCKET_CREATION_FAILED		= 63;
	public const int SP_NORESOURCES					= 64;
	public const int SP_BROADCAST_NOT_SUPPORTED		= 65;
	public const int SP_BAD_SERVER_MESSAGE			= 66;
	public const int SP_NO_SERVER_RUNNING			= 67;
	public const int SP_NO_NETWORK					= 68;
	public const int SP_NO_SERVER_RESPONSE			= 69;
	public const int SP_NO_LICENSE_AVAILABLE		= 70;
	public const int SP_INVALID_LICENSE				= 71;
	public const int SP_INVALID_OPERATION			= 72;
	public const int SP_BUFFER_TOO_SMALL			= 73;
	public const int SP_INTERNAL_ERROR				= 74;
	public const int SP_PACKET_ALREADY_INITIALIZED	= 75;
	public const int SP_PROTOCOL_NOT_INSTALLED		= 76;
	
	// String for OS Driver type

	public const int RB_WINNT_SYS_DRVR        = 5;          // Windows NT system driver
	public const int RB_WIN95_SYS_DRVR        = 7;          // Windows 95 system driver
	public const int RB_NW_LOCAL_DRVR         = 8;          // Netware local driver
	
	/* Maximum number of devices */
	public const int MAX_NUM_DEV   =10;

	/* Maximum host name length */
	public const int MAX_NAME_LEN = 64;
	
	/* Maximum host address length */
	public const int MAX_ADDR_LEN = 32;

	/* Making the LIcense Update Time programmable*/
	public const int MAX_HEARTBEAT =2592000;/* 30*24*60*60 seconds */
	public const int MIN_HEARTBEAT =60;/* 60 seconds */
	public const int INFINITE_HEARTBEAT =-1;

	public const int SPRO_MAX_QUERY_SIZE =56;//in bytes
		
	/*Communication Modes */
	public const string RNBO_STANDALONE       = "RNBO_STANDALONE";
	public const string RNBO_SPN_LOCAL        = "RNBO_SPN_LOCAL";
	public const string RNBO_SPN_DRIVER       = "RNBO_SPN_DRIVER";
	public const string RNBO_SPN_BROADCAST    = "RNBO_SPN_BROADCAST";
	public const string RNBO_SPN_ALL_MODES    = "RNBO_SPN_ALL_MODES";
	public const string RNBO_SPN_SERVER_MODES = "RNBO_SPN_SERVER_MODES";
	

	/*define API packet for communication   */
	public  const uint SPRO_APIPACKET_SIZE = 4112;
	public static byte[] packet = new byte[SPRO_APIPACKET_SIZE];
	
	/*Import SSP API from sx32w.dll  */
	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproFormatPacket")]
	public static extern ushort RNBOsproFormatPacket(/* IN */ byte[] packet,
													 /* IN */uint packetSize );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproInitialize")]
	public static extern ushort RNBOsproInitialize(/* IN */ byte[] packet);

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproFindFirstUnit")]
	public static extern ushort RNBOsproFindFirstUnit(/* IN */ byte[] packet, 
													  /* IN */ushort developerID);

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproFindNextUnit")]
	public static extern ushort RNBOsproFindNextUnit(/* IN */ byte[] packet );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproRead")]
	public static extern ushort RNBOsproRead( /* IN */byte[] packet, 
		                                      /* IN */ushort address,
											  /* OUT */ ref ushort data );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproExtendedRead")]
	public static extern ushort RNBOsproExtendedRead(/* IN */ byte[] packet,
													 /* IN */ ushort address,
													 /* OUT */ ref ushort data,
													 /* OUT */ ref byte accessCode);

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproWrite")]
	public static extern ushort RNBOsproWrite(/* IN */ byte[] packet, 
											  /* IN */ushort writePassword,
											  /* IN */ ushort address,
											  /* IN */ ushort data, 
											  /* IN*/byte accessCode );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproOverwrite")]
	public static extern ushort RNBOsproOverwrite(/* IN */ byte[] packet,
												  /* IN */ ushort writePassword, 
												  /* IN */ushort overwritePassword1, 
												  /* IN */ushort overwritePassword2,
												  /* IN */ ushort address, 
												  /* IN */ushort data,
												  /* IN */ byte accessCode );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproDecrement")]
	public static extern ushort RNBOsproDecrement( /* IN */byte[] packet,
												   /* IN */ ushort writePassword, 
												   /* IN */ushort address );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproActivate")]
	public static extern ushort RNBOsproActivate(/* IN */ byte[] packet, 
												 /* IN */ushort writePassword,
												 /* IN */ushort activatePassword1,
												 /* IN */ ushort activatePassword2,
												 ushort address );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproQuery")]
	public static extern ushort RNBOsproQuery(/* IN */ byte[] packet, 
											  /* IN*/ushort address,
											  /* IN */ byte[] queryData,
											  /* OUT */ byte[] response,
											  /* OUT */ ref uint response32, 
											  /* IN */ushort length );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproGetFullStatus")]
	public static extern ushort RNBOsproGetFullStatus( /* IN */byte[] packet );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproGetVersion")]
	public static extern ushort RNBOsproGetVersion(/* IN */ byte[] packet,
												   /* OUT */ref byte majVer,
												   /* OUT */ ref byte minVer,
												   /* OUT */ ref byte rev, 
												   /* OUT */ref byte osDrvrType );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproGetHardLimit")]
	public static extern ushort RNBOsproGetHardLimit( /* IN */byte[] packet,
													  /* OUT */ out ushort hardLimit );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproGetSubLicense")]
	public static extern ushort RNBOsproGetSubLicense (/* IN */ byte[]	packet, 
													   /* IN */ushort address );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproReleaseLicense")]
	public static extern ushort RNBOsproReleaseLicense (/* IN */ byte[] packet,
														/* IN */ ushort address, 
														/* IN/OUT */ref ushort numSubLic );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproSetContactServer")]
	public static extern ushort RNBOsproSetContactServer (/* IN */ byte[] packet,
														  /* IN */ [MarshalAs(UnmanagedType.LPStr)] string serverName );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproGetContactServer")]
	public static extern ushort RNBOsproGetContactServer (/* IN */ byte[] packet, /* OUT */byte[] serverNameBuf, 
														  /* IN */uint serverNameBufSz );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproEnumServer")]
	public static extern ushort RNBOsproEnumServer ( int enumFlag, ushort developerId, 
													 byte[] nsproServerInfo,
													 ref ushort numServerInfo );
	
	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproGetKeyInfo")]
	public static extern ushort RNBOsproGetKeyInfo (/* IN */ byte[] packet,
													/* IN */ ushort devId,	
													/* IN */ushort keyIndex,
													/* OUT */ byte[]  nsproMonitorInfo );

	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproSetProtocol")]
	public static extern ushort RNBOsproSetProtocol (/* IN */ byte[] packet,
													 /* IN */int protocol );
	
	[DllImport("sx32w.dll", CharSet=CharSet.Ansi, EntryPoint="RNBOsproSetHeartBeat")]
	public static extern ushort RNBOsproSetHeartBeat(/* IN */ byte[] packet,
													 /* IN */ int heartBeatValue );

}
